import 'liveConsultation.m.dart';

import 'package:fis_jsonrpc/utils.dart';

class AIDiagnosisRect {
	int right;
	int bottom;
	int left;
	int top;
	int width;
	int height;

	AIDiagnosisRect({
		this.right = 0,
		this.bottom = 0,
		this.left = 0,
		this.top = 0,
		this.width = 0,
		this.height = 0,
	});

	factory AIDiagnosisRect.fromJson(Map<String, dynamic> map) {
		return AIDiagnosisRect( 
			right: map['Right'],
			bottom: map['Bottom'],
			left: map['Left'],
			top: map['Top'],
			width: map['Width'],
			height: map['Height'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Right'] = right;
		map['Bottom'] = bottom;
		map['Left'] = left;
		map['Top'] = top;
		map['Width'] = width;
		map['Height'] = height;
		return map;
	}
}

class AIDiagnosisPoint2D {
	int x;
	int y;

	AIDiagnosisPoint2D({
		this.x = 0,
		this.y = 0,
	});

	factory AIDiagnosisPoint2D.fromJson(Map<String, dynamic> map) {
		return AIDiagnosisPoint2D( 
			x: map['X'],
			y: map['Y'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['X'] = x;
		map['Y'] = y;
		return map;
	}
}

enum DiagnosisDescriptionEnum {
	Shape,
	Orientation,
	EchoPattern,
	LesionBoundary,
	Margin,
	Calcification,
	LesionSize,
	ThyroidEchoPattern,
	ThyroidShape,
	ThyroidMargin,
	ThyroidEchogenicFoci,
	LiverShape,
	LiverBoundary,
	LiverEchoTexture,
}

class AIDiagnosisDescription {
	DiagnosisDescriptionEnum type;
	String? value;

	AIDiagnosisDescription({
		this.type = DiagnosisDescriptionEnum.Shape,
		this.value,
	});

	factory AIDiagnosisDescription.fromJson(Map<String, dynamic> map) {
		return AIDiagnosisDescription( 
			type: DiagnosisDescriptionEnum.values.firstWhere((e) => e.index == map['Type']),
			value: map['Value'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Type'] = type.index;
		if(value != null)
			map['Value'] = value;
		return map;
	}
}

class AIDetectedObject {
	int label;
	double confidence;
	AIDiagnosisRect? boundingBox;
	List<AIDiagnosisPoint2D >? contours;
	List<AIDiagnosisDescription >? descriptions;

	AIDetectedObject({
		this.label = 0,
		this.confidence = 0,
		this.boundingBox,
		this.contours,
		this.descriptions,
	});

	factory AIDetectedObject.fromJson(Map<String, dynamic> map) {
		return AIDetectedObject( 
			label: map['Label'],
			confidence: double.parse(map['Confidence'].toString()),
			boundingBox: map['BoundingBox'] != null ? AIDiagnosisRect.fromJson(map['BoundingBox']) : null,
			contours: map['Contours'] != null ? (map['Contours'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			descriptions: map['Descriptions'] != null ? (map['Descriptions'] as List).map((e)=>AIDiagnosisDescription.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Label'] = label;
		map['Confidence'] = confidence;
		if(boundingBox != null)
			map['BoundingBox'] = boundingBox;
		if(contours != null)
			map['Contours'] = contours;
		if(descriptions != null)
			map['Descriptions'] = descriptions;
		return map;
	}
}

class AIDiagnosisResultPerOrgan {
	DiagnosisOrganEnum organ;
	AIDiagnosisRect? organBoundBox;
	List<AIDiagnosisPoint2D >? organContours;
	List<AIDiagnosisDescription >? organDescriptions;
	List<AIDetectedObject >? detectedObjects;

	AIDiagnosisResultPerOrgan({
		this.organ = DiagnosisOrganEnum.Null,
		this.organBoundBox,
		this.organContours,
		this.organDescriptions,
		this.detectedObjects,
	});

	factory AIDiagnosisResultPerOrgan.fromJson(Map<String, dynamic> map) {
		return AIDiagnosisResultPerOrgan( 
			organ: DiagnosisOrganEnum.values.firstWhere((e) => e.index == map['Organ']),
			organBoundBox: map['OrganBoundBox'] != null ? AIDiagnosisRect.fromJson(map['OrganBoundBox']) : null,
			organContours: map['OrganContours'] != null ? (map['OrganContours'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			organDescriptions: map['OrganDescriptions'] != null ? (map['OrganDescriptions'] as List).map((e)=>AIDiagnosisDescription.fromJson(e as Map<String,dynamic>)).toList() : null,
			detectedObjects: map['DetectedObjects'] != null ? (map['DetectedObjects'] as List).map((e)=>AIDetectedObject.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Organ'] = organ.index;
		if(organBoundBox != null)
			map['OrganBoundBox'] = organBoundBox;
		if(organContours != null)
			map['OrganContours'] = organContours;
		if(organDescriptions != null)
			map['OrganDescriptions'] = organDescriptions;
		if(detectedObjects != null)
			map['DetectedObjects'] = detectedObjects;
		return map;
	}
}

class AIDiagnosisPerImageDTO {
	int index;
	double priorityScore;
	List<AIDiagnosisResultPerOrgan >? diagResultsForEachOrgan;

	AIDiagnosisPerImageDTO({
		this.index = 0,
		this.priorityScore = 0,
		this.diagResultsForEachOrgan,
	});

	factory AIDiagnosisPerImageDTO.fromJson(Map<String, dynamic> map) {
		return AIDiagnosisPerImageDTO( 
			index: map['Index'],
			priorityScore: double.parse(map['PriorityScore'].toString()),
			diagResultsForEachOrgan: map['DiagResultsForEachOrgan'] != null ? (map['DiagResultsForEachOrgan'] as List).map((e)=>AIDiagnosisResultPerOrgan.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Index'] = index;
		map['PriorityScore'] = priorityScore;
		if(diagResultsForEachOrgan != null)
			map['DiagResultsForEachOrgan'] = diagResultsForEachOrgan;
		return map;
	}
}

class DiagnosisImageResult {
	DiagnosisConclusionEnum diagnosisConclusion;
	List<AIDiagnosisPerImageDTO >? diagnosisResult;
	List<DiagnosisOrganEnum >? diagnosisOrgans;
	CarotidResultDTO? carotidResult;

	DiagnosisImageResult({
		this.diagnosisConclusion = DiagnosisConclusionEnum.NotRequired,
		this.diagnosisResult,
		this.diagnosisOrgans,
		this.carotidResult,
	});

	factory DiagnosisImageResult.fromJson(Map<String, dynamic> map) {
		return DiagnosisImageResult( 
			diagnosisConclusion: DiagnosisConclusionEnum.values.firstWhere((e) => e.index == map['DiagnosisConclusion']),
			diagnosisResult: map['DiagnosisResult'] != null ? (map['DiagnosisResult'] as List).map((e)=>AIDiagnosisPerImageDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			diagnosisOrgans: map['DiagnosisOrgans'] != null ? (map['DiagnosisOrgans'] as List).map((e)=>DiagnosisOrganEnum.values.firstWhere((i) => i.index == e)).toList() : null,
			carotidResult: map['CarotidResult'] != null ? CarotidResultDTO.fromJson(map['CarotidResult']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['DiagnosisConclusion'] = diagnosisConclusion.index;
		if(diagnosisResult != null)
			map['DiagnosisResult'] = diagnosisResult;
		if(diagnosisOrgans != null)
			map['DiagnosisOrgans'] = diagnosisOrgans;
		if(carotidResult != null)
			map['CarotidResult'] = carotidResult;
		return map;
	}
}

class DiagnosisImageRequest extends TokenRequest{
	String? fileToken;
	String? diskPath;
	String? relationCode;

	DiagnosisImageRequest({
		this.fileToken,
		this.diskPath,
		this.relationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory DiagnosisImageRequest.fromJson(Map<String, dynamic> map) {
		return DiagnosisImageRequest( 
			fileToken: map['FileToken'],
			diskPath: map['DiskPath'],
			relationCode: map['RelationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(fileToken != null)
			map['FileToken'] = fileToken;
		if(diskPath != null)
			map['DiskPath'] = diskPath;
		if(relationCode != null)
			map['RelationCode'] = relationCode;
		return map;
	}
}

class GetDiagnosisImageResultByAIResultRequest {
	String? fileUrl;
	String? relationCode;
	DateTime? createTime;
	DateTime? updateTime;
	String? aiResult;

	GetDiagnosisImageResultByAIResultRequest({
		this.fileUrl,
		this.relationCode,
		this.createTime,
		this.updateTime,
		this.aiResult,
	});

	factory GetDiagnosisImageResultByAIResultRequest.fromJson(Map<String, dynamic> map) {
		return GetDiagnosisImageResultByAIResultRequest( 
			fileUrl: map['FileUrl'],
			relationCode: map['RelationCode'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
			aiResult: map['AiResult'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(fileUrl != null)
			map['FileUrl'] = fileUrl;
		if(relationCode != null)
			map['RelationCode'] = relationCode;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(updateTime != null)
			map['UpdateTime'] = JsonRpcUtils.dateFormat(updateTime!);
		if(aiResult != null)
			map['AiResult'] = aiResult;
		return map;
	}
}

class GetCarotidResultRequest {
	String? remedicalCode;
	String? surfaceToken;
	String? mdlToken;
	CarotidScanTypeEnum carotidScanType;
	CarotidScanDirectionEnum carotidScanDirection;
	List<MeasureImageFileDTO >? measureImageFiles;
	String? measureResult;

	GetCarotidResultRequest({
		this.remedicalCode,
		this.surfaceToken,
		this.mdlToken,
		this.carotidScanType = CarotidScanTypeEnum.CarotidLeft,
		this.carotidScanDirection = CarotidScanDirectionEnum.TopToBottom,
		this.measureImageFiles,
		this.measureResult,
	});

	factory GetCarotidResultRequest.fromJson(Map<String, dynamic> map) {
		return GetCarotidResultRequest( 
			remedicalCode: map['RemedicalCode'],
			surfaceToken: map['SurfaceToken'],
			mdlToken: map['MdlToken'],
			carotidScanType: CarotidScanTypeEnum.values.firstWhere((e) => e.index == map['CarotidScanType']),
			carotidScanDirection: CarotidScanDirectionEnum.values.firstWhere((e) => e.index == map['CarotidScanDirection']),
			measureImageFiles: map['MeasureImageFiles'] != null ? (map['MeasureImageFiles'] as List).map((e)=>MeasureImageFileDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			measureResult: map['MeasureResult'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(remedicalCode != null)
			map['RemedicalCode'] = remedicalCode;
		if(surfaceToken != null)
			map['SurfaceToken'] = surfaceToken;
		if(mdlToken != null)
			map['MdlToken'] = mdlToken;
		map['CarotidScanType'] = carotidScanType.index;
		map['CarotidScanDirection'] = carotidScanDirection.index;
		if(measureImageFiles != null)
			map['MeasureImageFiles'] = measureImageFiles;
		if(measureResult != null)
			map['MeasureResult'] = measureResult;
		return map;
	}
}

class CreateCarotidSurfaceImagesResult {
	List<String >? surfaceImages;

	CreateCarotidSurfaceImagesResult({
		this.surfaceImages,
	});

	factory CreateCarotidSurfaceImagesResult.fromJson(Map<String, dynamic> map) {
		return CreateCarotidSurfaceImagesResult( 
			surfaceImages: map['SurfaceImages'] != null ? map['SurfaceImages'].cast<String>().toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(surfaceImages != null)
			map['SurfaceImages'] = surfaceImages;
		return map;
	}
}

class CreateCarotidSurfaceImagesRequest {
	String? surfaceFileUrl;

	CreateCarotidSurfaceImagesRequest({
		this.surfaceFileUrl,
	});

	factory CreateCarotidSurfaceImagesRequest.fromJson(Map<String, dynamic> map) {
		return CreateCarotidSurfaceImagesRequest( 
			surfaceFileUrl: map['SurfaceFileUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(surfaceFileUrl != null)
			map['SurfaceFileUrl'] = surfaceFileUrl;
		return map;
	}
}

class DiagnosisPerImageDTO extends AIDiagnosisPerImageDTO{
	String? remedicalCode;
	RemedicalFileDataTypeEnum dataType;
	double pixel;
	String? remedicalFileToken;
	String? aIFileToken;
	String? aIPreviewFileToken;
	String? perImageJson;
	DiagnosisConclusionEnum diagnosisConclusion;
	List<DiagnosisOrganEnum >? diagnosisOrgans;

	DiagnosisPerImageDTO({
		this.remedicalCode,
		this.dataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
		this.pixel = 0,
		this.remedicalFileToken,
		this.aIFileToken,
		this.aIPreviewFileToken,
		this.perImageJson,
		this.diagnosisConclusion = DiagnosisConclusionEnum.NotRequired,
		this.diagnosisOrgans,
		int index = 0,
		double priorityScore = 0,
		List<AIDiagnosisResultPerOrgan >? diagResultsForEachOrgan,
	}) : super(
			index: index,
			priorityScore: priorityScore,
			diagResultsForEachOrgan: diagResultsForEachOrgan,
		);

	factory DiagnosisPerImageDTO.fromJson(Map<String, dynamic> map) {
		return DiagnosisPerImageDTO( 
			remedicalCode: map['RemedicalCode'],
			dataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['DataType']),
			pixel: double.parse(map['Pixel'].toString()),
			remedicalFileToken: map['RemedicalFileToken'],
			aIFileToken: map['AIFileToken'],
			aIPreviewFileToken: map['AIPreviewFileToken'],
			perImageJson: map['PerImageJson'],
			diagnosisConclusion: DiagnosisConclusionEnum.values.firstWhere((e) => e.index == map['DiagnosisConclusion']),
			diagnosisOrgans: map['DiagnosisOrgans'] != null ? (map['DiagnosisOrgans'] as List).map((e)=>DiagnosisOrganEnum.values.firstWhere((i) => i.index == e)).toList() : null,
			index: map['Index'],
			priorityScore: double.parse(map['PriorityScore'].toString()),
			diagResultsForEachOrgan: map['DiagResultsForEachOrgan'] != null ? (map['DiagResultsForEachOrgan'] as List).map((e)=>AIDiagnosisResultPerOrgan.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(remedicalCode != null)
			map['RemedicalCode'] = remedicalCode;
		map['DataType'] = dataType.index;
		map['Pixel'] = pixel;
		if(remedicalFileToken != null)
			map['RemedicalFileToken'] = remedicalFileToken;
		if(aIFileToken != null)
			map['AIFileToken'] = aIFileToken;
		if(aIPreviewFileToken != null)
			map['AIPreviewFileToken'] = aIPreviewFileToken;
		if(perImageJson != null)
			map['PerImageJson'] = perImageJson;
		map['DiagnosisConclusion'] = diagnosisConclusion.index;
		if(diagnosisOrgans != null)
			map['DiagnosisOrgans'] = diagnosisOrgans;
		return map;
	}
}

class DiagnosisRemicalDTO {
	String? remedicalCode;
	RemedicalFileDataTypeEnum dataType;
	String? fileToken;
	List<AIDiagnosisPerImageDTO >? diagnosisResult;
	CarotidResultDTO? carotidResult;

	DiagnosisRemicalDTO({
		this.remedicalCode,
		this.dataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
		this.fileToken,
		this.diagnosisResult,
		this.carotidResult,
	});

	factory DiagnosisRemicalDTO.fromJson(Map<String, dynamic> map) {
		return DiagnosisRemicalDTO( 
			remedicalCode: map['RemedicalCode'],
			dataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['DataType']),
			fileToken: map['FileToken'],
			diagnosisResult: map['DiagnosisResult'] != null ? (map['DiagnosisResult'] as List).map((e)=>AIDiagnosisPerImageDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			carotidResult: map['CarotidResult'] != null ? CarotidResultDTO.fromJson(map['CarotidResult']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(remedicalCode != null)
			map['RemedicalCode'] = remedicalCode;
		map['DataType'] = dataType.index;
		if(fileToken != null)
			map['FileToken'] = fileToken;
		if(diagnosisResult != null)
			map['DiagnosisResult'] = diagnosisResult;
		if(carotidResult != null)
			map['CarotidResult'] = carotidResult;
		return map;
	}
}

class DiagnosisReportResult {
	DiagnosisConclusionEnum diagnosisConclusion;
	List<DiagnosisPerImageDTO >? diagnosisResult;
	List<DiagnosisRemicalDTO >? carotidResult;

	DiagnosisReportResult({
		this.diagnosisConclusion = DiagnosisConclusionEnum.NotRequired,
		this.diagnosisResult,
		this.carotidResult,
	});

	factory DiagnosisReportResult.fromJson(Map<String, dynamic> map) {
		return DiagnosisReportResult( 
			diagnosisConclusion: DiagnosisConclusionEnum.values.firstWhere((e) => e.index == map['DiagnosisConclusion']),
			diagnosisResult: map['DiagnosisResult'] != null ? (map['DiagnosisResult'] as List).map((e)=>DiagnosisPerImageDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			carotidResult: map['CarotidResult'] != null ? (map['CarotidResult'] as List).map((e)=>DiagnosisRemicalDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['DiagnosisConclusion'] = diagnosisConclusion.index;
		if(diagnosisResult != null)
			map['DiagnosisResult'] = diagnosisResult;
		if(carotidResult != null)
			map['CarotidResult'] = carotidResult;
		return map;
	}
}

class DiagnosisReportRequest extends TokenRequest{
	DiagnosisOrganEnum organ;
	List<DiagnosisRemicalDTO >? remedicalList;

	DiagnosisReportRequest({
		this.organ = DiagnosisOrganEnum.Null,
		this.remedicalList,
		String? token,
	}) : super(
			token: token,
		);

	factory DiagnosisReportRequest.fromJson(Map<String, dynamic> map) {
		return DiagnosisReportRequest( 
			organ: DiagnosisOrganEnum.values.firstWhere((e) => e.index == map['Organ']),
			remedicalList: map['RemedicalList'] != null ? (map['RemedicalList'] as List).map((e)=>DiagnosisRemicalDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['Organ'] = organ.index;
		if(remedicalList != null)
			map['RemedicalList'] = remedicalList;
		return map;
	}
}

class GetDiagnosisConclusionResult {
	DiagnosisConclusionEnum diagnosisConclusion;
	List<DiagnosisOrganEnum >? diagnosisOrgans;

	GetDiagnosisConclusionResult({
		this.diagnosisConclusion = DiagnosisConclusionEnum.NotRequired,
		this.diagnosisOrgans,
	});

	factory GetDiagnosisConclusionResult.fromJson(Map<String, dynamic> map) {
		return GetDiagnosisConclusionResult( 
			diagnosisConclusion: DiagnosisConclusionEnum.values.firstWhere((e) => e.index == map['DiagnosisConclusion']),
			diagnosisOrgans: map['DiagnosisOrgans'] != null ? (map['DiagnosisOrgans'] as List).map((e)=>DiagnosisOrganEnum.values.firstWhere((i) => i.index == e)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['DiagnosisConclusion'] = diagnosisConclusion.index;
		if(diagnosisOrgans != null)
			map['DiagnosisOrgans'] = diagnosisOrgans;
		return map;
	}
}

class GetDiagnosisConclusionRequest extends TokenRequest{
	String? perImageJson;

	GetDiagnosisConclusionRequest({
		this.perImageJson,
		String? token,
	}) : super(
			token: token,
		);

	factory GetDiagnosisConclusionRequest.fromJson(Map<String, dynamic> map) {
		return GetDiagnosisConclusionRequest( 
			perImageJson: map['PerImageJson'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(perImageJson != null)
			map['PerImageJson'] = perImageJson;
		return map;
	}
}

class EnumFieldDTO {
	int id;
	String? value;

	EnumFieldDTO({
		this.id = 0,
		this.value,
	});

	factory EnumFieldDTO.fromJson(Map<String, dynamic> map) {
		return EnumFieldDTO( 
			id: map['Id'],
			value: map['Value'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Id'] = id;
		if(value != null)
			map['Value'] = value;
		return map;
	}
}

class EnumItemDTO {
	String? code;
	List<EnumFieldDTO >? children;

	EnumItemDTO({
		this.code,
		this.children,
	});

	factory EnumItemDTO.fromJson(Map<String, dynamic> map) {
		return EnumItemDTO( 
			code: map['Code'],
			children: map['Children'] != null ? (map['Children'] as List).map((e)=>EnumFieldDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(code != null)
			map['Code'] = code;
		if(children != null)
			map['Children'] = children;
		return map;
	}
}

class GetDiagnosisEnumItemsResult {
	List<EnumItemDTO >? source;

	GetDiagnosisEnumItemsResult({
		this.source,
	});

	factory GetDiagnosisEnumItemsResult.fromJson(Map<String, dynamic> map) {
		return GetDiagnosisEnumItemsResult( 
			source: map['Source'] != null ? (map['Source'] as List).map((e)=>EnumItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(source != null)
			map['Source'] = source;
		return map;
	}
}

class GetDiagnosisEnumItemsRequest extends TokenRequest{

	GetDiagnosisEnumItemsRequest({
		String? token,
	}) : super(
			token: token,
		);

	factory GetDiagnosisEnumItemsRequest.fromJson(Map<String, dynamic> map) {
		return GetDiagnosisEnumItemsRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

enum DiagnosisKeyPointType {
	Null,
	OtherKeyPoints,
	HorizontalPointLeft,
	HorizontalPointRight,
	VerticalPointUp,
	VerticalPointDown,
}

class DiagnosisKeyPointDTO {
	DiagnosisKeyPointType type;
	AIDiagnosisPoint2D? point;
	int indexInContour;

	DiagnosisKeyPointDTO({
		this.type = DiagnosisKeyPointType.Null,
		this.point,
		this.indexInContour = 0,
	});

	factory DiagnosisKeyPointDTO.fromJson(Map<String, dynamic> map) {
		return DiagnosisKeyPointDTO( 
			type: DiagnosisKeyPointType.values.firstWhere((e) => e.index == map['Type']),
			point: map['Point'] != null ? AIDiagnosisPoint2D.fromJson(map['Point']) : null,
			indexInContour: map['IndexInContour'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Type'] = type.index;
		if(point != null)
			map['Point'] = point;
		map['IndexInContour'] = indexInContour;
		return map;
	}
}

class AIDiagnosisLesionSize {
	AIDiagnosisPoint2D? horizontalPoint1;
	AIDiagnosisPoint2D? horizontalPoint2;
	AIDiagnosisPoint2D? verticalPoint1;
	AIDiagnosisPoint2D? verticalPoint2;
	int horizontalLengthInPixel;
	int verticalLengthInPixel;

	AIDiagnosisLesionSize({
		this.horizontalPoint1,
		this.horizontalPoint2,
		this.verticalPoint1,
		this.verticalPoint2,
		this.horizontalLengthInPixel = 0,
		this.verticalLengthInPixel = 0,
	});

	factory AIDiagnosisLesionSize.fromJson(Map<String, dynamic> map) {
		return AIDiagnosisLesionSize( 
			horizontalPoint1: map['HorizontalPoint1'] != null ? AIDiagnosisPoint2D.fromJson(map['HorizontalPoint1']) : null,
			horizontalPoint2: map['HorizontalPoint2'] != null ? AIDiagnosisPoint2D.fromJson(map['HorizontalPoint2']) : null,
			verticalPoint1: map['VerticalPoint1'] != null ? AIDiagnosisPoint2D.fromJson(map['VerticalPoint1']) : null,
			verticalPoint2: map['VerticalPoint2'] != null ? AIDiagnosisPoint2D.fromJson(map['VerticalPoint2']) : null,
			horizontalLengthInPixel: map['HorizontalLengthInPixel'],
			verticalLengthInPixel: map['VerticalLengthInPixel'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(horizontalPoint1 != null)
			map['HorizontalPoint1'] = horizontalPoint1;
		if(horizontalPoint2 != null)
			map['HorizontalPoint2'] = horizontalPoint2;
		if(verticalPoint1 != null)
			map['VerticalPoint1'] = verticalPoint1;
		if(verticalPoint2 != null)
			map['VerticalPoint2'] = verticalPoint2;
		map['HorizontalLengthInPixel'] = horizontalLengthInPixel;
		map['VerticalLengthInPixel'] = verticalLengthInPixel;
		return map;
	}
}

class GetKeyPointsOfContourRequest extends TokenRequest{
	List<AIDiagnosisPoint2D >? contours;
	AIDiagnosisLesionSize? lesionSize;

	GetKeyPointsOfContourRequest({
		this.contours,
		this.lesionSize,
		String? token,
	}) : super(
			token: token,
		);

	factory GetKeyPointsOfContourRequest.fromJson(Map<String, dynamic> map) {
		return GetKeyPointsOfContourRequest( 
			contours: map['Contours'] != null ? (map['Contours'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			lesionSize: map['LesionSize'] != null ? AIDiagnosisLesionSize.fromJson(map['LesionSize']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(contours != null)
			map['Contours'] = contours;
		if(lesionSize != null)
			map['LesionSize'] = lesionSize;
		return map;
	}
}

class AffectedKeyPointsByDragActionRequest extends TokenRequest{
	List<DiagnosisKeyPointDTO >? keyPoints;
	AIDiagnosisPoint2D? mousePoint;

	AffectedKeyPointsByDragActionRequest({
		this.keyPoints,
		this.mousePoint,
		String? token,
	}) : super(
			token: token,
		);

	factory AffectedKeyPointsByDragActionRequest.fromJson(Map<String, dynamic> map) {
		return AffectedKeyPointsByDragActionRequest( 
			keyPoints: map['KeyPoints'] != null ? (map['KeyPoints'] as List).map((e)=>DiagnosisKeyPointDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			mousePoint: map['MousePoint'] != null ? AIDiagnosisPoint2D.fromJson(map['MousePoint']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(keyPoints != null)
			map['KeyPoints'] = keyPoints;
		if(mousePoint != null)
			map['MousePoint'] = mousePoint;
		return map;
	}
}

class ContourAndKeyPointsAfterDragResult {
	List<AIDiagnosisPoint2D >? dstContours;
	List<DiagnosisKeyPointDTO >? dstKeyPoints;
	List<int >? affectedKeyPointIndexes;

	ContourAndKeyPointsAfterDragResult({
		this.dstContours,
		this.dstKeyPoints,
		this.affectedKeyPointIndexes,
	});

	factory ContourAndKeyPointsAfterDragResult.fromJson(Map<String, dynamic> map) {
		return ContourAndKeyPointsAfterDragResult( 
			dstContours: map['DstContours'] != null ? (map['DstContours'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			dstKeyPoints: map['DstKeyPoints'] != null ? (map['DstKeyPoints'] as List).map((e)=>DiagnosisKeyPointDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			affectedKeyPointIndexes: map['AffectedKeyPointIndexes'] != null ? map['AffectedKeyPointIndexes'].cast<int>().toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(dstContours != null)
			map['DstContours'] = dstContours;
		if(dstKeyPoints != null)
			map['DstKeyPoints'] = dstKeyPoints;
		if(affectedKeyPointIndexes != null)
			map['AffectedKeyPointIndexes'] = affectedKeyPointIndexes;
		return map;
	}
}

class ContourAndKeyPointsAfterDragRequest extends TokenRequest{
	List<AIDiagnosisPoint2D >? contours;
	List<DiagnosisKeyPointDTO >? keyPoints;
	AIDiagnosisPoint2D? startPoint;
	AIDiagnosisPoint2D? endPoint;

	ContourAndKeyPointsAfterDragRequest({
		this.contours,
		this.keyPoints,
		this.startPoint,
		this.endPoint,
		String? token,
	}) : super(
			token: token,
		);

	factory ContourAndKeyPointsAfterDragRequest.fromJson(Map<String, dynamic> map) {
		return ContourAndKeyPointsAfterDragRequest( 
			contours: map['Contours'] != null ? (map['Contours'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			keyPoints: map['KeyPoints'] != null ? (map['KeyPoints'] as List).map((e)=>DiagnosisKeyPointDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			startPoint: map['StartPoint'] != null ? AIDiagnosisPoint2D.fromJson(map['StartPoint']) : null,
			endPoint: map['EndPoint'] != null ? AIDiagnosisPoint2D.fromJson(map['EndPoint']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(contours != null)
			map['Contours'] = contours;
		if(keyPoints != null)
			map['KeyPoints'] = keyPoints;
		if(startPoint != null)
			map['StartPoint'] = startPoint;
		if(endPoint != null)
			map['EndPoint'] = endPoint;
		return map;
	}
}

class MinimumDistanceToContourPointsResult {
	double distanceCaught;
	int closestPointIndex;

	MinimumDistanceToContourPointsResult({
		this.distanceCaught = 0,
		this.closestPointIndex = 0,
	});

	factory MinimumDistanceToContourPointsResult.fromJson(Map<String, dynamic> map) {
		return MinimumDistanceToContourPointsResult( 
			distanceCaught: double.parse(map['DistanceCaught'].toString()),
			closestPointIndex: map['ClosestPointIndex'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['DistanceCaught'] = distanceCaught;
		map['ClosestPointIndex'] = closestPointIndex;
		return map;
	}
}

class MinimumDistanceToContourPointsRequest extends TokenRequest{
	List<AIDiagnosisPoint2D >? contourPoints;
	AIDiagnosisPoint2D? mousePoint;

	MinimumDistanceToContourPointsRequest({
		this.contourPoints,
		this.mousePoint,
		String? token,
	}) : super(
			token: token,
		);

	factory MinimumDistanceToContourPointsRequest.fromJson(Map<String, dynamic> map) {
		return MinimumDistanceToContourPointsRequest( 
			contourPoints: map['ContourPoints'] != null ? (map['ContourPoints'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			mousePoint: map['MousePoint'] != null ? AIDiagnosisPoint2D.fromJson(map['MousePoint']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(contourPoints != null)
			map['ContourPoints'] = contourPoints;
		if(mousePoint != null)
			map['MousePoint'] = mousePoint;
		return map;
	}
}

class ContourMergeResult {
	List<AIDiagnosisPoint2D >? dstContours;
	AIDiagnosisLesionSize? dstLesionSize;

	ContourMergeResult({
		this.dstContours,
		this.dstLesionSize,
	});

	factory ContourMergeResult.fromJson(Map<String, dynamic> map) {
		return ContourMergeResult( 
			dstContours: map['DstContours'] != null ? (map['DstContours'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			dstLesionSize: map['DstLesionSize'] != null ? AIDiagnosisLesionSize.fromJson(map['DstLesionSize']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(dstContours != null)
			map['DstContours'] = dstContours;
		if(dstLesionSize != null)
			map['DstLesionSize'] = dstLesionSize;
		return map;
	}
}

class ContourMergeRequest extends TokenRequest{
	List<AIDiagnosisPoint2D >? contourPoints;
	AIDiagnosisLesionSize? lesionSize;
	List<AIDiagnosisPoint2D >? drawingNewContourPoints;

	ContourMergeRequest({
		this.contourPoints,
		this.lesionSize,
		this.drawingNewContourPoints,
		String? token,
	}) : super(
			token: token,
		);

	factory ContourMergeRequest.fromJson(Map<String, dynamic> map) {
		return ContourMergeRequest( 
			contourPoints: map['ContourPoints'] != null ? (map['ContourPoints'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			lesionSize: map['LesionSize'] != null ? AIDiagnosisLesionSize.fromJson(map['LesionSize']) : null,
			drawingNewContourPoints: map['DrawingNewContourPoints'] != null ? (map['DrawingNewContourPoints'] as List).map((e)=>AIDiagnosisPoint2D.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(contourPoints != null)
			map['ContourPoints'] = contourPoints;
		if(lesionSize != null)
			map['LesionSize'] = lesionSize;
		if(drawingNewContourPoints != null)
			map['DrawingNewContourPoints'] = drawingNewContourPoints;
		return map;
	}
}


