import 'liveConsultation.m.dart';
import 'notification.m.dart';

import 'package:fis_jsonrpc/utils.dart';

class GetUserInfoByOpenIdResult {
	String? nickName;
	String? hospital;
	String? userId;

	GetUserInfoByOpenIdResult({
		this.nickName,
		this.hospital,
		this.userId,
	});

	factory GetUserInfoByOpenIdResult.fromJson(Map<String, dynamic> map) {
		return GetUserInfoByOpenIdResult( 
			nickName: map['NickName'],
			hospital: map['Hospital'],
			userId: map['UserId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(nickName != null)
			map['NickName'] = nickName;
		if(hospital != null)
			map['Hospital'] = hospital;
		if(userId != null)
			map['UserId'] = userId;
		return map;
	}
}

class AppletAPIBaseRequest {
	String? openId;

	AppletAPIBaseRequest({
		this.openId,
	});

	factory AppletAPIBaseRequest.fromJson(Map<String, dynamic> map) {
		return AppletAPIBaseRequest( 
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(openId != null)
			map['OpenId'] = openId;
		return map;
	}
}

class GetUserInfoByOpenIdRequest extends AppletAPIBaseRequest{

	GetUserInfoByOpenIdRequest({
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetUserInfoByOpenIdRequest.fromJson(Map<String, dynamic> map) {
		return GetUserInfoByOpenIdRequest( 
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class BindVcloudAccountAndOpenIdRequest {
	String? account;
	String? password;
	String? openId;

	BindVcloudAccountAndOpenIdRequest({
		this.account,
		this.password,
		this.openId,
	});

	factory BindVcloudAccountAndOpenIdRequest.fromJson(Map<String, dynamic> map) {
		return BindVcloudAccountAndOpenIdRequest( 
			account: map['Account'],
			password: map['Password'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(account != null)
			map['Account'] = account;
		if(password != null)
			map['Password'] = password;
		if(openId != null)
			map['OpenId'] = openId;
		return map;
	}
}

enum RemedicalAIDiagnosisStatusEnum {
	Null,
	NoObviousLesion,
	Benign,
	Malignant,
	BenignAndMalignant,
}

class RemedicalRecordInfo {
	String? remedicalRecordId;
	String? terminalDesc;
	String? organizationDesc;
	DateTime? createTime;
	String? patientId;
	String? patientName;
	bool reportUploadStatus;
	RemedicalAIDiagnosisStatusEnum aIDiagnosisStatus;

	RemedicalRecordInfo({
		this.remedicalRecordId,
		this.terminalDesc,
		this.organizationDesc,
		this.createTime,
		this.patientId,
		this.patientName,
		this.reportUploadStatus = false,
		this.aIDiagnosisStatus = RemedicalAIDiagnosisStatusEnum.Null,
	});

	factory RemedicalRecordInfo.fromJson(Map<String, dynamic> map) {
		return RemedicalRecordInfo( 
			remedicalRecordId: map['RemedicalRecordId'],
			terminalDesc: map['TerminalDesc'],
			organizationDesc: map['OrganizationDesc'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			patientId: map['PatientId'],
			patientName: map['PatientName'],
			reportUploadStatus: map['ReportUploadStatus'],
			aIDiagnosisStatus: RemedicalAIDiagnosisStatusEnum.values.firstWhere((e) => e.index == map['AIDiagnosisStatus']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(remedicalRecordId != null)
			map['RemedicalRecordId'] = remedicalRecordId;
		if(terminalDesc != null)
			map['TerminalDesc'] = terminalDesc;
		if(organizationDesc != null)
			map['OrganizationDesc'] = organizationDesc;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(patientId != null)
			map['PatientId'] = patientId;
		if(patientName != null)
			map['PatientName'] = patientName;
		map['ReportUploadStatus'] = reportUploadStatus;
		map['AIDiagnosisStatus'] = aIDiagnosisStatus.index;
		return map;
	}
}

class AppletAPIPageRequest extends AppletAPIBaseRequest{
	int pageIndex;
	int pageSize;

	AppletAPIPageRequest({
		this.pageIndex = 0,
		this.pageSize = 0,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory AppletAPIPageRequest.fromJson(Map<String, dynamic> map) {
		return AppletAPIPageRequest( 
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['PageIndex'] = pageIndex;
		map['PageSize'] = pageSize;
		return map;
	}
}

class GetRemedicalRecordListRequest extends AppletAPIPageRequest{
	String? keyWord;
	DateTime? startTime;
	DateTime? endTime;

	GetRemedicalRecordListRequest({
		this.keyWord,
		this.startTime,
		this.endTime,
		int pageIndex = 0,
		int pageSize = 0,
		String? openId,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			openId: openId,
		);

	factory GetRemedicalRecordListRequest.fromJson(Map<String, dynamic> map) {
		return GetRemedicalRecordListRequest( 
			keyWord: map['KeyWord'],
			startTime: map['StartTime'] != null ? DateTime.parse(map['StartTime']) : null,
			endTime: map['EndTime'] != null ? DateTime.parse(map['EndTime']) : null,
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(keyWord != null)
			map['KeyWord'] = keyWord;
		if(startTime != null)
			map['StartTime'] = JsonRpcUtils.dateFormat(startTime!);
		if(endTime != null)
			map['EndTime'] = JsonRpcUtils.dateFormat(endTime!);
		return map;
	}
}

class ImageFile {
	String? id;
	String? fileUrl;
	DateTime? createTime;

	ImageFile({
		this.id,
		this.fileUrl,
		this.createTime,
	});

	factory ImageFile.fromJson(Map<String, dynamic> map) {
		return ImageFile( 
			id: map['Id'],
			fileUrl: map['FileUrl'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		if(fileUrl != null)
			map['FileUrl'] = fileUrl;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		return map;
	}
}

class ImageData {
	String? id;
	RemedicalFileDataTypeEnum dataType;
	String? application;
	String? previewFileToken;
	DateTime? createTime;
	ImageFile? file;

	ImageData({
		this.id,
		this.dataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
		this.application,
		this.previewFileToken,
		this.createTime,
		this.file,
	});

	factory ImageData.fromJson(Map<String, dynamic> map) {
		return ImageData( 
			id: map['Id'],
			dataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['DataType']),
			application: map['Application'],
			previewFileToken: map['PreviewFileToken'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			file: map['File'] != null ? ImageFile.fromJson(map['File']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		map['DataType'] = dataType.index;
		if(application != null)
			map['Application'] = application;
		if(previewFileToken != null)
			map['PreviewFileToken'] = previewFileToken;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(file != null)
			map['File'] = file;
		return map;
	}
}

class GetRemedicalDataListRequest extends AppletAPIBaseRequest{
	String? remedicalRecordId;

	GetRemedicalDataListRequest({
		this.remedicalRecordId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetRemedicalDataListRequest.fromJson(Map<String, dynamic> map) {
		return GetRemedicalDataListRequest( 
			remedicalRecordId: map['RemedicalRecordId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(remedicalRecordId != null)
			map['RemedicalRecordId'] = remedicalRecordId;
		return map;
	}
}

enum AppletAPIImageType {
	placeHolder_0,
	Local,
	Ob,
	WorkSheet,
	Remote,
	DigitalSignature,
	AIDiagnosis,
}

class ReportImageInfo {
	String? examDataId;
	String? fileUrl;
	AppletAPIImageType type;

	ReportImageInfo({
		this.examDataId,
		this.fileUrl,
		this.type = AppletAPIImageType.Local,
	});

	factory ReportImageInfo.fromJson(Map<String, dynamic> map) {
		return ReportImageInfo( 
			examDataId: map['ExamDataId'],
			fileUrl: map['FileUrl'],
			type: AppletAPIImageType.values.firstWhere((e) => e.index == map['Type']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(examDataId != null)
			map['ExamDataId'] = examDataId;
		if(fileUrl != null)
			map['FileUrl'] = fileUrl;
		map['Type'] = type.index;
		return map;
	}
}

class ReportInfo {
	String? id;
	List<ReportImageInfo >? previewImages;

	ReportInfo({
		this.id,
		this.previewImages,
	});

	factory ReportInfo.fromJson(Map<String, dynamic> map) {
		return ReportInfo( 
			id: map['Id'],
			previewImages: map['PreviewImages'] != null ? (map['PreviewImages'] as List).map((e)=>ReportImageInfo.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		if(previewImages != null)
			map['PreviewImages'] = previewImages;
		return map;
	}
}

class GetRemedicalReportListRequest extends AppletAPIBaseRequest{
	String? remedicalRecordId;

	GetRemedicalReportListRequest({
		this.remedicalRecordId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetRemedicalReportListRequest.fromJson(Map<String, dynamic> map) {
		return GetRemedicalReportListRequest( 
			remedicalRecordId: map['RemedicalRecordId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(remedicalRecordId != null)
			map['RemedicalRecordId'] = remedicalRecordId;
		return map;
	}
}

enum ConsultationState {
	Unhandled,
	Withdrawn,
	Handled,
	Started,
	WaitForReportUpload,
	ReportUploading,
	ReportUploaded,
	ReportUploadFailed,
	Expired,
	Qualified,
	UnQualified,
}

enum PatientGenderEnum {
	NotFilled,
	Male,
	Female,
}

class ImageTokenInfo {
	String? imageToken;
	String? previewToken;

	ImageTokenInfo({
		this.imageToken,
		this.previewToken,
	});

	factory ImageTokenInfo.fromJson(Map<String, dynamic> map) {
		return ImageTokenInfo( 
			imageToken: map['ImageToken'],
			previewToken: map['PreviewToken'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(imageToken != null)
			map['ImageToken'] = imageToken;
		if(previewToken != null)
			map['PreviewToken'] = previewToken;
		return map;
	}
}

class VideoTokenInfo {
	String? videoToken;
	String? previewToken;

	VideoTokenInfo({
		this.videoToken,
		this.previewToken,
	});

	factory VideoTokenInfo.fromJson(Map<String, dynamic> map) {
		return VideoTokenInfo( 
			videoToken: map['VideoToken'],
			previewToken: map['PreviewToken'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(videoToken != null)
			map['VideoToken'] = videoToken;
		if(previewToken != null)
			map['PreviewToken'] = previewToken;
		return map;
	}
}

class ReportPreviewPdf {
	String? reportId;
	DateTime? updateTime;
	String? previewPdfUrl;

	ReportPreviewPdf({
		this.reportId,
		this.updateTime,
		this.previewPdfUrl,
	});

	factory ReportPreviewPdf.fromJson(Map<String, dynamic> map) {
		return ReportPreviewPdf( 
			reportId: map['ReportId'],
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
			previewPdfUrl: map['PreviewPdfUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(reportId != null)
			map['ReportId'] = reportId;
		if(updateTime != null)
			map['UpdateTime'] = JsonRpcUtils.dateFormat(updateTime!);
		if(previewPdfUrl != null)
			map['PreviewPdfUrl'] = previewPdfUrl;
		return map;
	}
}

class ConsultationInfo {
	String? consultationId;
	ConsultationState state;
	String? patientName;
	PatientGenderEnum patientSex;
	String? patientAge;
	DateTime? birthday;
	String? contactInfo;
	String? patientID;
	String? history;
	String? primaryDiagnosis;
	String? hospitalId;
	String? hospitalName;
	String? applyHospitalId;
	String? applyHospitalName;
	String? expertId;
	String? expertName;
	DateTime? consultationDate;
	String? inspectionItems;
	String? scanDoctorName;
	String? deviceModel;
	String? terminalId;
	List<ImageTokenInfo >? imageTokens;
	List<VideoTokenInfo >? videoInfos;
	List<ReportPreviewPdf >? reportPreviewPdfs;

	ConsultationInfo({
		this.consultationId,
		this.state = ConsultationState.Unhandled,
		this.patientName,
		this.patientSex = PatientGenderEnum.NotFilled,
		this.patientAge,
		this.birthday,
		this.contactInfo,
		this.patientID,
		this.history,
		this.primaryDiagnosis,
		this.hospitalId,
		this.hospitalName,
		this.applyHospitalId,
		this.applyHospitalName,
		this.expertId,
		this.expertName,
		this.consultationDate,
		this.inspectionItems,
		this.scanDoctorName,
		this.deviceModel,
		this.terminalId,
		this.imageTokens,
		this.videoInfos,
		this.reportPreviewPdfs,
	});

	factory ConsultationInfo.fromJson(Map<String, dynamic> map) {
		return ConsultationInfo( 
			consultationId: map['ConsultationId'],
			state: ConsultationState.values.firstWhere((e) => e.index == map['State']),
			patientName: map['PatientName'],
			patientSex: PatientGenderEnum.values.firstWhere((e) => e.index == map['PatientSex']),
			patientAge: map['PatientAge'],
			birthday: map['Birthday'] != null ? DateTime.parse(map['Birthday']) : null,
			contactInfo: map['ContactInfo'],
			patientID: map['PatientID'],
			history: map['History'],
			primaryDiagnosis: map['PrimaryDiagnosis'],
			hospitalId: map['HospitalId'],
			hospitalName: map['HospitalName'],
			applyHospitalId: map['ApplyHospitalId'],
			applyHospitalName: map['ApplyHospitalName'],
			expertId: map['ExpertId'],
			expertName: map['ExpertName'],
			consultationDate: map['ConsultationDate'] != null ? DateTime.parse(map['ConsultationDate']) : null,
			inspectionItems: map['InspectionItems'],
			scanDoctorName: map['ScanDoctorName'],
			deviceModel: map['DeviceModel'],
			terminalId: map['TerminalId'],
			imageTokens: map['ImageTokens'] != null ? (map['ImageTokens'] as List).map((e)=>ImageTokenInfo.fromJson(e as Map<String,dynamic>)).toList() : null,
			videoInfos: map['VideoInfos'] != null ? (map['VideoInfos'] as List).map((e)=>VideoTokenInfo.fromJson(e as Map<String,dynamic>)).toList() : null,
			reportPreviewPdfs: map['ReportPreviewPdfs'] != null ? (map['ReportPreviewPdfs'] as List).map((e)=>ReportPreviewPdf.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationId != null)
			map['ConsultationId'] = consultationId;
		map['State'] = state.index;
		if(patientName != null)
			map['PatientName'] = patientName;
		map['PatientSex'] = patientSex.index;
		if(patientAge != null)
			map['PatientAge'] = patientAge;
		if(birthday != null)
			map['Birthday'] = JsonRpcUtils.dateFormat(birthday!);
		if(contactInfo != null)
			map['ContactInfo'] = contactInfo;
		if(patientID != null)
			map['PatientID'] = patientID;
		if(history != null)
			map['History'] = history;
		if(primaryDiagnosis != null)
			map['PrimaryDiagnosis'] = primaryDiagnosis;
		if(hospitalId != null)
			map['HospitalId'] = hospitalId;
		if(hospitalName != null)
			map['HospitalName'] = hospitalName;
		if(applyHospitalId != null)
			map['ApplyHospitalId'] = applyHospitalId;
		if(applyHospitalName != null)
			map['ApplyHospitalName'] = applyHospitalName;
		if(expertId != null)
			map['ExpertId'] = expertId;
		if(expertName != null)
			map['ExpertName'] = expertName;
		if(consultationDate != null)
			map['ConsultationDate'] = JsonRpcUtils.dateFormat(consultationDate!);
		if(inspectionItems != null)
			map['InspectionItems'] = inspectionItems;
		if(scanDoctorName != null)
			map['ScanDoctorName'] = scanDoctorName;
		if(deviceModel != null)
			map['DeviceModel'] = deviceModel;
		if(terminalId != null)
			map['TerminalId'] = terminalId;
		if(imageTokens != null)
			map['ImageTokens'] = imageTokens;
		if(videoInfos != null)
			map['VideoInfos'] = videoInfos;
		if(reportPreviewPdfs != null)
			map['ReportPreviewPdfs'] = reportPreviewPdfs;
		return map;
	}
}

class GetConsultationListRequest extends AppletAPIPageRequest{
	String? keyWord;
	DateTime? startTime;
	DateTime? endTime;

	GetConsultationListRequest({
		this.keyWord,
		this.startTime,
		this.endTime,
		int pageIndex = 0,
		int pageSize = 0,
		String? openId,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			openId: openId,
		);

	factory GetConsultationListRequest.fromJson(Map<String, dynamic> map) {
		return GetConsultationListRequest( 
			keyWord: map['KeyWord'],
			startTime: map['StartTime'] != null ? DateTime.parse(map['StartTime']) : null,
			endTime: map['EndTime'] != null ? DateTime.parse(map['EndTime']) : null,
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(keyWord != null)
			map['KeyWord'] = keyWord;
		if(startTime != null)
			map['StartTime'] = JsonRpcUtils.dateFormat(startTime!);
		if(endTime != null)
			map['EndTime'] = JsonRpcUtils.dateFormat(endTime!);
		return map;
	}
}

class GetConsultationReportListRequest extends AppletAPIBaseRequest{
	String? consultationId;

	GetConsultationReportListRequest({
		this.consultationId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetConsultationReportListRequest.fromJson(Map<String, dynamic> map) {
		return GetConsultationReportListRequest( 
			consultationId: map['ConsultationId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationId != null)
			map['ConsultationId'] = consultationId;
		return map;
	}
}

class CreateConsultationRequest extends AppletAPIBaseRequest{
	String? patientName;
	PatientGenderEnum patientSex;
	String? patientAge;
	DateTime? birthday;
	String? contactInfo;
	String? patientID;
	String? history;
	String? primaryDiagnosis;
	String? hospitalId;
	String? expertId;
	DateTime? consultationDate;
	String? inspectionItems;
	String? scanDoctorName;
	String? deviceModel;
	String? terminalId;

	CreateConsultationRequest({
		this.patientName,
		this.patientSex = PatientGenderEnum.NotFilled,
		this.patientAge,
		this.birthday,
		this.contactInfo,
		this.patientID,
		this.history,
		this.primaryDiagnosis,
		this.hospitalId,
		this.expertId,
		this.consultationDate,
		this.inspectionItems,
		this.scanDoctorName,
		this.deviceModel,
		this.terminalId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory CreateConsultationRequest.fromJson(Map<String, dynamic> map) {
		return CreateConsultationRequest( 
			patientName: map['PatientName'],
			patientSex: PatientGenderEnum.values.firstWhere((e) => e.index == map['PatientSex']),
			patientAge: map['PatientAge'],
			birthday: map['Birthday'] != null ? DateTime.parse(map['Birthday']) : null,
			contactInfo: map['ContactInfo'],
			patientID: map['PatientID'],
			history: map['History'],
			primaryDiagnosis: map['PrimaryDiagnosis'],
			hospitalId: map['HospitalId'],
			expertId: map['ExpertId'],
			consultationDate: map['ConsultationDate'] != null ? DateTime.parse(map['ConsultationDate']) : null,
			inspectionItems: map['InspectionItems'],
			scanDoctorName: map['ScanDoctorName'],
			deviceModel: map['DeviceModel'],
			terminalId: map['TerminalId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientName != null)
			map['PatientName'] = patientName;
		map['PatientSex'] = patientSex.index;
		if(patientAge != null)
			map['PatientAge'] = patientAge;
		if(birthday != null)
			map['Birthday'] = JsonRpcUtils.dateFormat(birthday!);
		if(contactInfo != null)
			map['ContactInfo'] = contactInfo;
		if(patientID != null)
			map['PatientID'] = patientID;
		if(history != null)
			map['History'] = history;
		if(primaryDiagnosis != null)
			map['PrimaryDiagnosis'] = primaryDiagnosis;
		if(hospitalId != null)
			map['HospitalId'] = hospitalId;
		if(expertId != null)
			map['ExpertId'] = expertId;
		if(consultationDate != null)
			map['ConsultationDate'] = JsonRpcUtils.dateFormat(consultationDate!);
		if(inspectionItems != null)
			map['InspectionItems'] = inspectionItems;
		if(scanDoctorName != null)
			map['ScanDoctorName'] = scanDoctorName;
		if(deviceModel != null)
			map['DeviceModel'] = deviceModel;
		if(terminalId != null)
			map['TerminalId'] = terminalId;
		return map;
	}
}

class ArrangeConsultationRequest extends AppletAPIBaseRequest{
	String? consultationId;
	DateTime? appointDate;
	String? expertId;

	ArrangeConsultationRequest({
		this.consultationId,
		this.appointDate,
		this.expertId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory ArrangeConsultationRequest.fromJson(Map<String, dynamic> map) {
		return ArrangeConsultationRequest( 
			consultationId: map['ConsultationId'],
			appointDate: map['AppointDate'] != null ? DateTime.parse(map['AppointDate']) : null,
			expertId: map['ExpertId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationId != null)
			map['ConsultationId'] = consultationId;
		if(appointDate != null)
			map['AppointDate'] = JsonRpcUtils.dateFormat(appointDate!);
		if(expertId != null)
			map['ExpertId'] = expertId;
		return map;
	}
}

class OrganBaseInfo {
	String? organizationId;
	String? name;

	OrganBaseInfo({
		this.organizationId,
		this.name,
	});

	factory OrganBaseInfo.fromJson(Map<String, dynamic> map) {
		return OrganBaseInfo( 
			organizationId: map['OrganizationId'],
			name: map['Name'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(organizationId != null)
			map['OrganizationId'] = organizationId;
		if(name != null)
			map['Name'] = name;
		return map;
	}
}

class GetHospitalsRequest extends AppletAPIBaseRequest{

	GetHospitalsRequest({
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetHospitalsRequest.fromJson(Map<String, dynamic> map) {
		return GetHospitalsRequest( 
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class DoctorInfo {
	String? id;
	String? displayName;
	String? fullName;
	String? electronSignUrl;

	DoctorInfo({
		this.id,
		this.displayName,
		this.fullName,
		this.electronSignUrl,
	});

	factory DoctorInfo.fromJson(Map<String, dynamic> map) {
		return DoctorInfo( 
			id: map['Id'],
			displayName: map['DisplayName'],
			fullName: map['FullName'],
			electronSignUrl: map['ElectronSignUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		if(displayName != null)
			map['DisplayName'] = displayName;
		if(fullName != null)
			map['FullName'] = fullName;
		if(electronSignUrl != null)
			map['ElectronSignUrl'] = electronSignUrl;
		return map;
	}
}

class GetExpertsRequest extends AppletAPIBaseRequest{
	String? organizationId;

	GetExpertsRequest({
		this.organizationId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetExpertsRequest.fromJson(Map<String, dynamic> map) {
		return GetExpertsRequest( 
			organizationId: map['OrganizationId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationId != null)
			map['OrganizationId'] = organizationId;
		return map;
	}
}

class TerminalInfo {
	String? id;
	String? organizationId;
	String? organizationDecription;
	String? organizationName;
	String? name;
	String? model;
	String? description;
	String? uniqueId;

	TerminalInfo({
		this.id,
		this.organizationId,
		this.organizationDecription,
		this.organizationName,
		this.name,
		this.model,
		this.description,
		this.uniqueId,
	});

	factory TerminalInfo.fromJson(Map<String, dynamic> map) {
		return TerminalInfo( 
			id: map['Id'],
			organizationId: map['OrganizationId'],
			organizationDecription: map['OrganizationDecription'],
			organizationName: map['OrganizationName'],
			name: map['Name'],
			model: map['Model'],
			description: map['Description'],
			uniqueId: map['UniqueId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		if(organizationId != null)
			map['OrganizationId'] = organizationId;
		if(organizationDecription != null)
			map['OrganizationDecription'] = organizationDecription;
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		if(name != null)
			map['Name'] = name;
		if(model != null)
			map['Model'] = model;
		if(description != null)
			map['Description'] = description;
		if(uniqueId != null)
			map['UniqueId'] = uniqueId;
		return map;
	}
}

class GetTerminalListRequest extends AppletAPIBaseRequest{

	GetTerminalListRequest({
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetTerminalListRequest.fromJson(Map<String, dynamic> map) {
		return GetTerminalListRequest( 
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class GetScanLocationsRequest extends AppletAPIBaseRequest{

	GetScanLocationsRequest({
		String? openId,
	}) : super(
			openId: openId,
		);

	factory GetScanLocationsRequest.fromJson(Map<String, dynamic> map) {
		return GetScanLocationsRequest( 
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class LiveTerminalInfo2 {
	bool terminalLiveEnabled;
	bool cameraLiveEnabled;
	bool isMergeChannel;
	int terminalWidth;
	int terminalHeight;
	String? cameraId;
	bool isMultiChannels;
	int cameraWidth;
	int cameraHeight;
	int integerRoomId;
	int terminalIntegerRoomId;
	LiveStates state;
	String? id;

	LiveTerminalInfo2({
		this.terminalLiveEnabled = false,
		this.cameraLiveEnabled = false,
		this.isMergeChannel = false,
		this.terminalWidth = 0,
		this.terminalHeight = 0,
		this.cameraId,
		this.isMultiChannels = false,
		this.cameraWidth = 0,
		this.cameraHeight = 0,
		this.integerRoomId = 0,
		this.terminalIntegerRoomId = 0,
		this.state = LiveStates.OK,
		this.id,
	});

	factory LiveTerminalInfo2.fromJson(Map<String, dynamic> map) {
		return LiveTerminalInfo2( 
			terminalLiveEnabled: map['TerminalLiveEnabled'],
			cameraLiveEnabled: map['CameraLiveEnabled'],
			isMergeChannel: map['IsMergeChannel'],
			terminalWidth: map['TerminalWidth'],
			terminalHeight: map['TerminalHeight'],
			cameraId: map['CameraId'],
			isMultiChannels: map['IsMultiChannels'],
			cameraWidth: map['CameraWidth'],
			cameraHeight: map['CameraHeight'],
			integerRoomId: map['IntegerRoomId'],
			terminalIntegerRoomId: map['TerminalIntegerRoomId'],
			state: LiveStates.values.firstWhere((e) => e.index == map['State']),
			id: map['Id'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['TerminalLiveEnabled'] = terminalLiveEnabled;
		map['CameraLiveEnabled'] = cameraLiveEnabled;
		map['IsMergeChannel'] = isMergeChannel;
		map['TerminalWidth'] = terminalWidth;
		map['TerminalHeight'] = terminalHeight;
		if(cameraId != null)
			map['CameraId'] = cameraId;
		map['IsMultiChannels'] = isMultiChannels;
		map['CameraWidth'] = cameraWidth;
		map['CameraHeight'] = cameraHeight;
		map['IntegerRoomId'] = integerRoomId;
		map['TerminalIntegerRoomId'] = terminalIntegerRoomId;
		map['State'] = state.index;
		if(id != null)
			map['Id'] = id;
		return map;
	}
}

class MeetingMemberInfo2 {
	LiveStates state;
	String? id;
	String? name;
	String? displayName;
	String? avatar;
	String? userSign;

	MeetingMemberInfo2({
		this.state = LiveStates.OK,
		this.id,
		this.name,
		this.displayName,
		this.avatar,
		this.userSign,
	});

	factory MeetingMemberInfo2.fromJson(Map<String, dynamic> map) {
		return MeetingMemberInfo2( 
			state: LiveStates.values.firstWhere((e) => e.index == map['State']),
			id: map['Id'],
			name: map['Name'],
			displayName: map['DisplayName'],
			avatar: map['Avatar'],
			userSign: map['UserSign'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['State'] = state.index;
		if(id != null)
			map['Id'] = id;
		if(name != null)
			map['Name'] = name;
		if(displayName != null)
			map['DisplayName'] = displayName;
		if(avatar != null)
			map['Avatar'] = avatar;
		if(userSign != null)
			map['UserSign'] = userSign;
		return map;
	}
}

class StartConsultationResult {
	String? roomId;
	LiveStates state;
	int integerRoomId;
	int appId;
	List<LiveTerminalInfo2 >? terminalInfos;
	List<MeetingMemberInfo2 >? meetingMemberInfos;

	StartConsultationResult({
		this.roomId,
		this.state = LiveStates.OK,
		this.integerRoomId = 0,
		this.appId = 0,
		this.terminalInfos,
		this.meetingMemberInfos,
	});

	factory StartConsultationResult.fromJson(Map<String, dynamic> map) {
		return StartConsultationResult( 
			roomId: map['RoomId'],
			state: LiveStates.values.firstWhere((e) => e.index == map['State']),
			integerRoomId: map['IntegerRoomId'],
			appId: map['AppId'],
			terminalInfos: map['TerminalInfos'] != null ? (map['TerminalInfos'] as List).map((e)=>LiveTerminalInfo2.fromJson(e as Map<String,dynamic>)).toList() : null,
			meetingMemberInfos: map['MeetingMemberInfos'] != null ? (map['MeetingMemberInfos'] as List).map((e)=>MeetingMemberInfo2.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(roomId != null)
			map['RoomId'] = roomId;
		map['State'] = state.index;
		map['IntegerRoomId'] = integerRoomId;
		map['AppId'] = appId;
		if(terminalInfos != null)
			map['TerminalInfos'] = terminalInfos;
		if(meetingMemberInfos != null)
			map['MeetingMemberInfos'] = meetingMemberInfos;
		return map;
	}
}

class StartConsultationRequest extends AppletAPIBaseRequest{
	String? consultationId;

	StartConsultationRequest({
		this.consultationId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory StartConsultationRequest.fromJson(Map<String, dynamic> map) {
		return StartConsultationRequest( 
			consultationId: map['ConsultationId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationId != null)
			map['ConsultationId'] = consultationId;
		return map;
	}
}

class CancelStartConsultationResult {
	LiveStates liveStates;

	CancelStartConsultationResult({
		this.liveStates = LiveStates.OK,
	});

	factory CancelStartConsultationResult.fromJson(Map<String, dynamic> map) {
		return CancelStartConsultationResult( 
			liveStates: LiveStates.values.firstWhere((e) => e.index == map['LiveStates']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['LiveStates'] = liveStates.index;
		return map;
	}
}

class CancelStartConsultationRequest extends AppletAPIBaseRequest{
	String? roomId;

	CancelStartConsultationRequest({
		this.roomId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory CancelStartConsultationRequest.fromJson(Map<String, dynamic> map) {
		return CancelStartConsultationRequest( 
			roomId: map['RoomId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(roomId != null)
			map['RoomId'] = roomId;
		return map;
	}
}

class ExitConsultationResult {
	LiveStates liveStates;

	ExitConsultationResult({
		this.liveStates = LiveStates.OK,
	});

	factory ExitConsultationResult.fromJson(Map<String, dynamic> map) {
		return ExitConsultationResult( 
			liveStates: LiveStates.values.firstWhere((e) => e.index == map['LiveStates']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['LiveStates'] = liveStates.index;
		return map;
	}
}

class ExitConsultationRequest extends AppletAPIBaseRequest{
	String? roomId;
	String? consultationId;

	ExitConsultationRequest({
		this.roomId,
		this.consultationId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory ExitConsultationRequest.fromJson(Map<String, dynamic> map) {
		return ExitConsultationRequest( 
			roomId: map['RoomId'],
			consultationId: map['ConsultationId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(roomId != null)
			map['RoomId'] = roomId;
		if(consultationId != null)
			map['ConsultationId'] = consultationId;
		return map;
	}
}

class ConsultationHeartRateRequest extends AppletAPIBaseRequest{
	String? consultationId;

	ConsultationHeartRateRequest({
		this.consultationId,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory ConsultationHeartRateRequest.fromJson(Map<String, dynamic> map) {
		return ConsultationHeartRateRequest( 
			consultationId: map['ConsultationId'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationId != null)
			map['ConsultationId'] = consultationId;
		return map;
	}
}

class AppletAPIDownloadFileResult {
	String? fileString;

	AppletAPIDownloadFileResult({
		this.fileString,
	});

	factory AppletAPIDownloadFileResult.fromJson(Map<String, dynamic> map) {
		return AppletAPIDownloadFileResult( 
			fileString: map['FileString'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(fileString != null)
			map['FileString'] = fileString;
		return map;
	}
}

class DownloadFileRequest extends AppletAPIBaseRequest{
	String? fileToken;

	DownloadFileRequest({
		this.fileToken,
		String? openId,
	}) : super(
			openId: openId,
		);

	factory DownloadFileRequest.fromJson(Map<String, dynamic> map) {
		return DownloadFileRequest( 
			fileToken: map['FileToken'],
			openId: map['OpenId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(fileToken != null)
			map['FileToken'] = fileToken;
		return map;
	}
}


